/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.aop;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.aop.InstanceAdvisor;
import org.jboss.aop.Advised;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.proxy.ClassProxy;
import org.jboss.aop.proxy.ClassProxyFactory;
import org.jboss.aop.util.MethodHashing;
import org.jboss.util.NestedRuntimeException;
import org.jboss.cache.Fqn;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.io.Serializable;

/**
 * Unitlity methods for put, get and remove Collection classes object.
 *
 * @author Ben Wang
 */
public class AopUtil
{
   static Log log=LogFactory.getLog(AopUtil.class.getName());

   /**
    * Find cache interceptor with exact fqn.
    * @param advisor
    * @param fqn
    * @return
    */
   static Interceptor findCacheInterceptor(InstanceAdvisor advisor, Fqn fqn)
   {
      org.jboss.aop.advice.Interceptor[] interceptors = advisor.getInterceptors();
      // Step Check for cross references
      for (int i = 0; i < interceptors.length; i++) {
         Interceptor interceptor = interceptors[i];
         if (interceptor instanceof CacheInterceptor) {
            CacheInterceptor inter = (CacheInterceptor)interceptor;
            if (inter != null && inter.getFqn().equals(fqn))
            {
               return interceptor;
            }
         }
      }
      return null;
   }

   /**
    * Find existing cache interceptor. Since there is supposedly only one cache interceptor per
    * pojo, this call should suffice. In addition, in cases of cross or circular reference,
    * fqn can be different anyway.
    * @param advisor
    * @return
    */
   static Interceptor findCacheInterceptor(InstanceAdvisor advisor)
   {
      // TODO we assume there is only one interceptor now.
      Interceptor[] interceptors = advisor.getInterceptors();
      // Step Check for cross references
      for (int i = 0; i < interceptors.length; i++) {
         Interceptor interceptor = interceptors[i];
         if (interceptor instanceof CacheInterceptor) {
               return interceptor;
         }
      }
      return null;
   }

   /**
    * Check whether the object type is valid. An object type is valid if it is either: aspectized,
    * Serializable, or primitive type. Otherwise a runtime exception is thrown.
    *
    * @param obj
    */
   static void checkObjectType(Object obj) {
      if(obj == null) return;
      if( ! (obj instanceof Advised) ) {
          if( !(obj instanceof Serializable ) ) {
               throw new IllegalArgumentException("TreeCacheAop.putObject(): Object type is neither " +
                  " aspectized nor Serializable. Object class name is " +obj.getClass().getName());
          }
      }
   }

}
