package org.jboss.cache.interceptors;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.TreeCache;
import org.jgroups.blocks.MethodCall;

import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.Transaction;

/**
 * Class representing an interceptor.
 * <em>Note that this will be replaced by {@link org.jboss.aop.advice.Interceptor} in one of the next releases</em>
 * @author Bela Ban
 * @version $Id: Interceptor.java,v 1.2 2005/04/05 17:19:26 belaban Exp $
 */
public abstract class Interceptor {
   Interceptor next=null;
   TreeCache   cache=null;
   Log         log=null;

   public Interceptor() {
      log=LogFactory.getLog(getClass());
   }


   public void setNext(Interceptor i) {
      next=i;
   }

   public Interceptor getNext() {
      return next;
   }

   public void setCache(TreeCache cache) {
      this.cache=cache;
   }

   public Object invoke(MethodCall m) throws Throwable {
      return next.invoke(m);
   }

   /** Returns true if transaction is ACTIVE, false otherwise */
   boolean isActive(Transaction tx) {
      if(tx == null) return false;
      int status=-1;
      try {
         status=tx.getStatus();
         return status == Status.STATUS_ACTIVE;
      }
      catch(SystemException e) {
         log.error("failed getting transaction status", e);
         return false;
      }
   }

   /** Returns true if transaction is PREPARING, false otherwise */
   boolean isPreparing(Transaction tx) {
      if(tx == null) return false;
      int status=-1;
      try {
         status=tx.getStatus();
         return status == Status.STATUS_PREPARING;
      }
      catch(SystemException e) {
         log.error("failed getting transaction status", e);
         return false;
      }
   }


   /**
    * Return s true of tx's status is ACTIVE or PREPARING
    * @param tx
    * @return
    */
   boolean isValid(Transaction tx) {
      return isActive(tx) || isPreparing(tx);
   }
}
