package org.jboss.cache.loader.rmi;

import org.jboss.cache.PropertyConfigurator;
import org.jboss.cache.TreeCache;

import java.rmi.Naming;

/**
 * Server which exports an RMI stub to the TreeCache. Clients can use the RmiDelegatingCacheLoader to remotely
 * delegate to this TreeCache
 * @author Bela Ban
 * @version $Id: RmiCacheServer.java,v 1.1 2005/04/28 11:22:55 bela Exp $
 */
public class RmiCacheServer {
   TreeCache cache;
   RemoteTreeCacheImpl remoteObj;
   String host;
   int port;
   String configFile;
   final String SUFFIX="CACHE_SERVER";

   public RmiCacheServer(String host, int port, String config) {
      this.host=host;
      this.port=port;
      this.configFile=config;
   }

   public void start() throws Exception {
      cache=new TreeCache();
      PropertyConfigurator config=new PropertyConfigurator();
      config.configure(cache, configFile);
      cache.createService();
      cache.startService(); // kick start tree cache
      remoteObj=new RemoteTreeCacheImpl(cache);
      Naming.rebind("//" + host + ":" + port +"/" + SUFFIX, remoteObj);
   }

   public void stop() {
      if(cache != null) {
         cache.stopService();
         cache.destroyService();
         cache=null;
      }
      if(remoteObj != null) {
         try {
            Naming.unbind("//" + host + ":" + port +"/" + SUFFIX);
         }
         catch(Exception e) {
            e.printStackTrace();
         }
      }
   }


   public static void main(String[] args) {
      String host="localhost", configFile="cache-service.xml";
      int port=1098;
      RmiCacheServer server;

      for(int i=0; i < args.length; i++) {
         if(args[i].equals("-host")) {
            host=args[++i];
            continue;
         }
         if(args[i].equals("-port")) {
            port=Integer.parseInt(args[+i]);
            continue;
         }
         if(args[i].equals("-config")) {
            configFile=args[++i];
            continue;
         }
         help();
         return;
      }
      server=new RmiCacheServer(host, port, configFile);
      try {
         server.start();
      }
      catch(Exception e) {
         e.printStackTrace();
      }
   }



   private static void help() {
      System.out.println("CacheServer [-host <host>] [-port <port>] [-config <cache config>][-help]");
   }


}
